source veclib.tcl

puts "\nvec3-Tests\n"

set a { 2 3 5 }
set b { 7 11 13 }
set s 17

puts "a = {$a}, b = {$b}, s = {$s}"

puts "a + b: [vec3+vec3 $a $b]"
puts "a - b: [vec3-vec3 $a $b]"
puts "a * b: [vec3*vec3 $a $b]"
puts "a * s: [vec3*s $a $s]"
puts "s * a: [s*vec3 $s $a]"
puts "a / s: [vec3/s $a $s]"
puts "a ^ b: [vec3^vec3 $a $b]"
puts "a & b: [vec3&vec3 $a $b]"
puts "|a|^2: [vec3normquad $a]"
puts "|a|: [vec3norm $a]"
puts "a/|a|: [vec3unit $a]"


puts "\nvec2-Tests\n"

set a { 2 3 }
set b { 5 7 }
set s 11

puts "a = {$a}, b = {$b}, s = {$s}"

puts "a + b: [vec2+vec2 $a $b]"
puts "a - b: [vec2-vec2 $a $b]"
puts "a * b: [vec2*vec2 $a $b]"
puts "a * s: [vec2*s $a $s]"
puts "s * a: [s*vec2 $s $a]"
puts "a / s: [vec2/s $a $s]"
puts "a ^ b: [vec2^vec2 $a $b]"
puts "a & b: [vec2&vec2 $a $b]"
puts "|a|^2: [vec2normquad $a]"
puts "|a|: [vec2norm $a]"
puts "a/|a|: [vec2unit $a]"


puts "\nquat-Tests\n"

set a { 2  { 3 5 7 } }
set b { 11 { 13 17 19 } }
set s 23

puts "a = {$a}, b = {$b}, s = {$s}"

puts "a + b: [quat+quat $a $b]"
puts "a - b: [quat-quat $a $b]"
puts "a * s: [quat*s $a $s]"
puts "s * a: [s*quat $s $a]"
puts "a / s: [quat/s $a $s]"
puts "a ^ b: [quat^quat $a $b]"
puts "|a|^2: [quatnormquad $a]"
puts "|a|: [quatnorm $a]"
puts "a/|a|: [quatunit $a]"
puts "*a: [quatconj $a]"
puts "1/a: [quatinvert $a]"

puts "\nsrt2-Tests\n"

set v {2 3}
set m {{1 5} {6 7}}
set a {5 7}
set b {{9 1} {3 2}}
puts "m = $m"
puts "v = $v"
puts "a = $a"
set l [srt=vecmat $v $m]
puts "l = (v,m) = $l"
puts "l * a = [srt2*vec2 $l $a]"
puts "l * b = [srt2*mat2 $l $b]"
set k [srt=vecmat $a $b]
puts "k = (a,b) = $k"
puts "k * l = [srt2*srt2 $k $l]"
set linv [srt2invert $l]
puts "linv = $linv"
puts "linv * l = [srt2*srt2 $linv $l]" 

puts "\nCheck matrix representation of srt2\n"
set lmat [mat3=srt2 $l]
puts "lmat = mat3=srt2 \$l = $lmat"
set kmat [mat3=srt2 $k]
puts "kmat = mat3=srt2 \$l = $kmat"
set lkmat [mat3=srt2 [srt2*srt2 $l $k]]
puts "lkmat = mat3=srt2 \$l*k = $lkmat"
puts "lmat * kmat = [mat3*mat3 $lmat $kmat]"

puts "\nsrt3-Tests\n"

set v {2 3 -1}
set m {{1 5 -7} {6 7 -3} {4 -4 8}}
set a {5 7 1}
set b {{9 1 -2} {-5 3 2} {11 17 13}}
puts "m = $m"
puts "v = $v"
puts "a = $a"
set l [srt=vecmat $v $m]
puts "l = (v,m) = $l"
puts "l * a = [srt3*vec3 $l $a]"
puts "l * b = [srt3*mat3 $l $b]"
set k [srt=vecmat $a $b]
puts "k = (a,b) = $k"
puts "k * l = [srt3*srt3 $k $l]"
set linv [srt3invert $l]
puts "linv = $linv"
puts "linv * l = [srt3*srt3 $linv $l]" 

puts "\nCheck matrix representation of srt3\n"
set lmat [mat4=srt3 $l]
puts "lmat = mat4=srt3 \$l = $lmat"
set kmat [mat4=srt3 $k]
puts "kmat = mat4=srt3 \$l = $kmat"
set lkmat [mat4=srt3 [srt3*srt3 $l $k]]
puts "lkmat = mat4=srt3 \$l*k = $lkmat"
puts "lmat * kmat = [mat4*mat4 $lmat $kmat]"

puts "\nmat2-Tests\n"

set a {{1 2} {3 4}}
set b {{10 20} {30 40}}
puts "a = $a\nb = $b"
puts "a + b = [mat2+mat2 $a $b]"
puts "a - b = [mat2-mat2 $a $b]"
puts "-a = [-mat2 $a]"
puts "det(a) = [mat2det $a]"

puts "\nTest mat2-rotations\n"
set r [mat2rotmatrix 0.123]
puts "r = mat2rotmatrix 0.123 = $r"
set w [mat2rotangle $r ]
puts "w = mat3rotangles \$r = $w"

puts "\nmat3-Tests\n"

set a {{1 2 3} {4 5 6} {7 8 9}}
set b {{10 20 30} {40 50 60} {70 80 90}}
puts "a = $a\nb = $b"
puts "a + b = [mat3+mat3 $a $b]"
puts "a - b = [mat3-mat3 $a $b]"
puts "-a = [-mat3 $a]"
puts "det(a) = [mat3det $a]"

puts "\nTest mat3-rotations\n"
set r [mat3rotmatrix 0 1 2 0.1 0.2 0.3]
puts "r = mat3rotmatrix 0 1 2 0.1 0.2 0.3 = $r"
set w [mat3rotangles $r 0 1 2 ]
puts "w = mat3rotangles \$r 0 1 2 = $w"

puts "\nTest mat3-vec3-duality\n"
set a {5 7 9}
puts "a = $a"
puts "dual(a) = vec3dual \$a = [vec3dual $a]"
puts "dual(dual(a)) = mat3dual \[vec3dual \$a\] = [mat3dual [vec3dual $a]]"








