# vecmat.tcl

proc vget { a i } {
	return [lindex $a $i]
	}

proc vset { _a i s } {
	upvar $_a a
	set a [lreplace $a $i $i $s]
	return $a
	}

proc mgetrow { a i } {
	return [lindex $a $i]
	}

proc msetrow { _a i v } {
	upvar $_a a
	set a [lreplace $a $i $i $v]
	return $a
	}

proc mget { a i j } {
	return [vget [mgetrow $a $i] $j]
	}

proc mset { _a i j s } {
	upvar $_a a
	set a [msetrow a $i [lreplace [mgetrow $a $i] $j $j $s]]
	return $a
	}

proc qget { a i } {
	if { $i == 0 } { return [vget $a 0] }
	return [mget $a 1 [expr $i - 1]]
	}

proc qset { _a i s } {
	upvar $_a a
	if { $i == 0 } { vset a 0 $s } \
	else { mset a 1 [expr $i - 1] $s }
	return $a
	}

proc qgets { a } {
	return [vget $a 0]
	}

proc qsets { _a s } {
	upvar $_a a
	set a [lreplace $a 0 0 $s]
	return $a
	}

proc qgetvec { a } {
	return [vget $a 1]
	}

proc qsetvec { _a v } {
	upvar $_a a
	set a [lreplace $a 1 1 $v]
	return $a
	}

proc srtgetvec { a } {
	return [vget $a 0]
	}

proc srtsetvec { _a v } {
	upvar $_a a
	vset a 0 $v
	return $a
	}

proc srtgetmat { a } {
	return [vget $a 1]
	}

proc srtsetmat { _a m } {
	upvar $_a a
	vset a 1 $m
	return $a
	}

# vec2.tcl

proc vec2+vec2 { a b } {
	return [list	[expr [vget $a 0] + [vget $b 0] ] \
			[expr [vget $a 1] + [vget $b 1] ] ]
	}

proc vec2-vec2 { a b } {
	return [list	[expr [vget $a 0] - [vget $b 0] ] \
			[expr [vget $a 1] - [vget $b 1] ] ]
	}

proc -vec2 { a } {
	return [list	[expr - [vget $a 0] ] \
			[expr - [vget $a 1] ] ]
	}

proc vec2*vec2 { a b } {
	return [expr	[vget $a 0] * [vget $b 0] + \
			[vget $a 1] * [vget $b 1] ]
	}

proc vec2*s { a b } {
	return [list	[expr [vget $a 0] * $b] \
			[expr [vget $a 1] * $b] ]
	}

proc s*vec2 { b a } {
	return [vec2*s $a $b]
	}

proc vec2/s { a b } {
	return [list	[expr [vget $a 0] / [expr double($b)]] \
		  	[expr [vget $a 1] / [expr double($b)]] ]
	}

proc vec2^vec2 { a b } {
	return [expr [vget $a 0] * [vget $b 1] - [vget $a 1] * [vget $b 0]]
	}

proc vec2&vec2 { a b } {
	return [list	[list	[expr [vget $a 0 ] * [vget $b 0]] \
				[expr [vget $a 0 ] * [vget $b 1]]]\
			[list	[expr [vget $a 1 ] * [vget $b 0]] \
				[expr [vget $a 1 ] * [vget $b 1]]]]
	}

proc vec2normquad { a } {
	return [vec2*vec2 $a $a]
	}

proc vec2norm { a } {
	return [sqrt [vec2*vec2 $a $a] ]
	}

proc vec2unit { a } {
	set n [vec2norm $a]
	if {$n == 0.0} {
		puts "vec2unit: cannot normalize 0-vector"
		return $a
		}
	return [vec2/s $a $n]
	}

proc isvec2 { a } {
	if { [llength $a] != 2 } {
		return 0
		}
	return 1
	}

# vec3.tcl

proc vec3+vec3 { a b } {
	return [list	[expr [vget $a 0] + [vget $b 0] ] \
			[expr [vget $a 1] + [vget $b 1] ] \
			[expr [vget $a 2] + [vget $b 2] ] ]
	}

proc vec3-vec3 { a b } {
	return [list	[expr [vget $a 0] - [vget $b 0] ] \
			[expr [vget $a 1] - [vget $b 1] ] \
			[expr [vget $a 2] - [vget $b 2] ] ]
	}

proc -vec3 { a } {
	return [list	[expr - [vget $a 0] ] \
			[expr - [vget $a 1] ] \
			[expr - [vget $a 2] ] ]
	}

proc vec3*vec3 { a b } {
	return [expr	[vget $a 0] * [vget $b 0] + \
			[vget $a 1] * [vget $b 1] + \
			[vget $a 2] * [vget $b 2] ]
	}

proc vec3*s { a b } {
	return [list	[expr [vget $a 0] * $b] \
			[expr [vget $a 1] * $b] \
			[expr [vget $a 2] * $b] ]
	}

proc s*vec3 { b a } {
	return [vec3*s $a $b]
	}

proc vec3/s { a b } {
	return [list	[expr [vget $a 0] / [expr double($b)]] \
		  	[expr [vget $a 1] / [expr double($b)]] \
		  	[expr [vget $a 2] / [expr double($b)]] ]
	}

proc vec3^vec3 { a b } {
	return [list	[expr [vget $a 1] * [vget $b 2] - [vget $a 2] * [vget $b 1]] \
			[expr [vget $a 2] * [vget $b 0] - [vget $a 0] * [vget $b 2]] \
			[expr [vget $a 0] * [vget $b 1] - [vget $a 1] * [vget $b 0]] ]
	}

proc vec3&vec3 { a b } {
	return [list	[list	[expr [vget $a 0 ] * [vget $b 0]] \
				[expr [vget $a 0 ] * [vget $b 1]] \
				[expr [vget $a 0 ] * [vget $b 2]]]\
			[list	[expr [vget $a 1 ] * [vget $b 0]] \
				[expr [vget $a 1 ] * [vget $b 1]] \
				[expr [vget $a 1 ] * [vget $b 2]]]\
			[list	[expr [vget $a 2 ] * [vget $b 0]] \
				[expr [vget $a 2 ] * [vget $b 1]] \
				[expr [vget $a 2 ] * [vget $b 2]]]]
	}

proc vec3normquad { a } {
	return [vec3*vec3 $a $a]
	}

proc vec3norm { a } {
	return [sqrt [vec3*vec3 $a $a] ]
	}

proc vec3unit { a } {
	set n [vec3norm $a]
	if {$n == 0.0} {
		puts "vec3unit: cannot normalize 0-vector"
		return $a
		}
	return [vec3/s $a $n]
	}

proc vec3dual { a } {
	return	[list	[list	0 [vget $a 2] [expr -[vget $a 1]]] \
			[list	[expr -[vget $a 2]] 0 [vget $a 0]] \
			[list	[vget $a 1] [expr -[vget $a 0]] 0] ]
	}

proc isvec3 { a } {
	if { [llength $a] != 3 } {
		return 0
		}
	return 1
	}

# quat.tcl

proc quat+quat { a b } {
	return [list	[expr [vget $a 0] + [vget $b 0]] \
			[vec3+vec3 [vget $a 1] [vget $b 1]]]
	}

proc quat-quat { a b } {
	return [list	[expr [vget $a 0] - [vget $b 0]] \
			[vec3-vec3 [vget $a 1] [vget $b 1]]]
	}


proc -quat { a } {
	return [list	[expr - [vget $a 0]] \
			[-vec3 [vget $a 1]]]
	}

proc quat*quat { a b } {
	return [expr	[vget $a 0] * [vget $b 0] + \
			[vec3*vec3 [vget $a 1] [vget $b 1]]]
	}

proc quat^quat { a b } {
	return [list	[expr [vget $a 0] * [vget $b 0] - [vec3*vec3 [vget $a 1] [vget $b 1]]] \
			[vec3+vec3 [vec3+vec3 [vec3*s [vget $b 1] [vget $a 0]] [vec3*s [vget $a 1] [vget $b 0]]] [vec3^vec3 [vget $a 1] [vget $b 1]]]]
	}

proc quat*s { a b } {
	return [list	[expr [vget $a 0] * $b] \
			[vec3*s [vget $a 1] $b]]
	}
	

proc s*quat { b a } {
	return [quat*s $a $b]
	}

proc quat/s { a b } {
	return [list	[expr [vget $a 0] / [expr double($b)]] \
			[vec3/s [vget $a 1] [expr double($b)]]]
	}

proc quat=s { a } {
	return [list $a [list 0 0 0]]
	}

proc quat=vec3 { a } {
	return [list 0 $a]
	}
	
proc quatnormquad { a } {
	return [expr [vget $a 0] * [vget $a 0] + [vec3*vec3 [vget $a 1] [vget $a 1]]]
	}

proc quatnorm { a } {
	return [expr sqrt( [quatnormquad $a] ) ]
	}

proc quatunit { a } {
	set n [quatnorm $a]
	if {$n == 0.0} {
		puts "quatunit: cannot normalize 0-quaternion"
		return $a
		}
	return [quat/s $a $n]
	}

proc quatconj { a } {
	return [list	[expr [vget $a 0]] \
			[-vec3 [vget $a 1]]]
	}
	
proc quatinvert { a } {
	return [quat/s [quatconj $a] [quatnormquad $a]]
	}

proc quatmaptomat3 { q } {
	set v [qgetvec $q]
	set s [qgets $q]
	set d [quatnormquad $q]

	set ma [mat3=s [expr $d - [expr 2.0 * [vec3normquad $v]]]]
	set mb [mat3*s [vec3&vec3 $v $v] 2.0]
	set mc [vec3dual [vec3*s $v [expr 2.0 * $s]]]

	return [mat3+mat3 $ma [mat3-mat3 $mb $mc]]
	}

# mat2.tcl	

proc mat2+mat2 { a b } {
	return [list	[vec2+vec2 [mgetrow $a 0] [mgetrow $b 0] ] \
			[vec2+vec2 [mgetrow $a 1] [mgetrow $b 1] ] ]
	}

proc mat2-mat2 { a b } {
	return [list	[vec2-vec2 [mgetrow $a 0] [mgetrow $b 0] ] \
			[vec2-vec2 [mgetrow $a 1] [mgetrow $b 1] ] ]
	}

proc -mat2 { a } {
	return [list	[-vec2 [mgetrow $a 0] ] \
			[-vec2 [mgetrow $a 1] ] ]
	}

proc mat2*mat2 { a b } {
	return [list	[list	[expr	  [mget $a 0 0] * [mget $b 0 0] \
					+ [mget $a 0 1] * [mget $b 1 0] ] \
				[expr	  [mget $a 0 0] * [mget $b 0 1] \
					+ [mget $a 0 1] * [mget $b 1 1] ] ] \
			[list	[expr	  [mget $a 1 0] * [mget $b 0 0] \
					+ [mget $a 1 1] * [mget $b 1 0] ] \
				[expr	  [mget $a 1 0] * [mget $b 0 1] \
					+ [mget $a 1 1] * [mget $b 1 1] ] ] ]
	}

proc mat2*s { a b } {
	return [list	[vec2*s [mgetrow $a 0] $b] \
			[vec2*s [mgetrow $a 1] $b] ]
	}

proc s*mat2 { b a } {
	return [mat2*s $a $b]
	}

proc mat2*vec2 { a b } {
	return [list 	[vec2*vec2 [mgetrow $a 0] $b] \
			[vec2*vec2 [mgetrow $a 1] $b] ]
	}

proc mat2/s { a b } {
	return [list	[vec2/s [mgetrow $a 0] $b] \
			[vec2/s [mgetrow $a 1] $b] ]
	}

proc mat2=s { b } {
	return [ list [list $b 0] [list 0 $b] ]
	}

proc mat2=diag { b00 b11 } {
	return [ list [list $b00 0] [list 0 $b11] ]
	}

proc mat2trace { a } {
	return 	[expr	[mget $a 0 0] + \
			[mget $a 1 1] ] \
	}

proc mat2det { a } {
	return [expr	  [mget $a 0 0] * [mget $a 1 1] \
			- [mget $a 0 1] * [mget $a 1 0] ]
	}

proc mat2trans { a } {
	return [list	[list	[mget $a 0 0] [mget $a 1 0] ] \
			[list	[mget $a 0 1] [mget $a 1 1] ] ]
	}

proc mat2subcyc { a i j } {
	return [mget $a [expr ($i + 1) % 2] [expr ($j + 1) % 2] ]
	}

proc mat2invert { a } {
	set d [mat2det $a]
	if {$d == 0.0} {
		puts "mat2invert: cannot invert degenerate matrix $a"
		return $a
		}
	set adj	[list	[list	[expr ([mget $a 1 1]) / double($d) ] \
				[expr (-[mget $a 0 1]) / double($d) ] ] \
			[list	[expr (-[mget $a 1 0]) / double($d) ] \
				[expr ([mget $a 0 0]) / double($d) ] ] ]
	return $adj
	}

proc mat2rotangle { a } {
	return [expr atan2([mget $a 1 0],[mget $a 0 0])]
	}

proc mat2rotmatrix { w } {
	set c [expr cos($w)]
	set s [expr sin($w)]

	return [list [list $c [expr -$s] ] [list $s $c] ]
	}

proc ismat2 { a } {
	if { [llength $a] != 2 } {
		return 0
		}
	if { ! [isvec2 [mgetrow $a 0]] } {
		return 0
		}
	if { ! [isvec2 [mgetrow $a 1]] } {
		return 0
		}
	return 1
	}

# mat3.tcl

proc mat3+mat3 { a b } {
	return [list	[vec3+vec3 [mgetrow $a 0] [mgetrow $b 0] ] \
			[vec3+vec3 [mgetrow $a 1] [mgetrow $b 1] ] \
			[vec3+vec3 [mgetrow $a 2] [mgetrow $b 2] ] ]
	}

proc mat3-mat3 { a b } {
	return [list	[vec3-vec3 [mgetrow $a 0] [mgetrow $b 0] ] \
			[vec3-vec3 [mgetrow $a 1] [mgetrow $b 1] ] \
			[vec3-vec3 [mgetrow $a 2] [mgetrow $b 2] ] ]
	}

proc -mat3 { a } {
	return [list	[-vec3 [mgetrow $a 0] ] \
			[-vec3 [mgetrow $a 1] ] \
			[-vec3 [mgetrow $a 2] ] ]
	}

proc mat3*mat3 { a b } {
	return [list	[list	[expr	  [mget $a 0 0] * [mget $b 0 0] \
					+ [mget $a 0 1] * [mget $b 1 0] \
					+ [mget $a 0 2] * [mget $b 2 0] ] \
				[expr	  [mget $a 0 0] * [mget $b 0 1] \
					+ [mget $a 0 1] * [mget $b 1 1] \
					+ [mget $a 0 2] * [mget $b 2 1] ] \
				[expr	  [mget $a 0 0] * [mget $b 0 2] \
					+ [mget $a 0 1] * [mget $b 1 2] \
					+ [mget $a 0 2] * [mget $b 2 2] ] ] \
			[list	[expr	  [mget $a 1 0] * [mget $b 0 0] \
					+ [mget $a 1 1] * [mget $b 1 0] \
					+ [mget $a 1 2] * [mget $b 2 0] ] \
				[expr	  [mget $a 1 0] * [mget $b 0 1] \
					+ [mget $a 1 1] * [mget $b 1 1] \
					+ [mget $a 1 2] * [mget $b 2 1] ] \
				[expr	  [mget $a 1 0] * [mget $b 0 2] \
					+ [mget $a 1 1] * [mget $b 1 2] \
					+ [mget $a 1 2] * [mget $b 2 2] ] ] \
			[list	[expr	  [mget $a 2 0] * [mget $b 0 0] \
					+ [mget $a 2 1] * [mget $b 1 0] \
					+ [mget $a 2 2] * [mget $b 2 0] ] \
				[expr	  [mget $a 2 0] * [mget $b 0 1] \
					+ [mget $a 2 1] * [mget $b 1 1] \
					+ [mget $a 2 2] * [mget $b 2 1] ] \
				[expr	  [mget $a 2 0] * [mget $b 0 2] \
					+ [mget $a 2 1] * [mget $b 1 2] \
					+ [mget $a 2 2] * [mget $b 2 2] ] ] ] \
	}

proc mat3*s { a b } {
	return [list	[vec3*s [mgetrow $a 0] $b] \
			[vec3*s [mgetrow $a 1] $b] \
			[vec3*s [mgetrow $a 2] $b] ]
	}

proc s*mat3 { b a } {
	return [mat3*s $a $b]
	}

proc mat3*vec3 { a b } {
	return [list 	[vec3*vec3 [mgetrow $a 0] $b] \
			[vec3*vec3 [mgetrow $a 1] $b] \
			[vec3*vec3 [mgetrow $a 2] $b] ]
	}

proc mat3/s { a b } {
	return [list	[vec3/s [mgetrow $a 0] $b] \
			[vec3/s [mgetrow $a 1] $b] \
			[vec3/s [mgetrow $a 2] $b] ]
	}

proc mat3=s { b } {
	return [ list [list $b 0 0] [list 0 $b 0] [list 0 0 $b] ]
	}

proc mat3=diag { b00 b11 b22 } {
	return [ list [list $b00 0 0] [list 0 $b11 0] [list 0 0 $b22] ]
	}

proc mat3trace { a } {
	return 	[expr	[mget $a 0 0] + \
			[mget $a 1 1] + \
			[mget $a 2 2] ]
	}

proc mat3trans { a } {
	return [list	[list	[mget $a 0 0] [mget $a 1 0] [mget $a 2 0] ] \
			[list	[mget $a 0 1] [mget $a 1 1] [mget $a 2 1] ] \
			[list	[mget $a 0 2] [mget $a 1 2] [mget $a 2 2] ] ]
	}

proc mat3det { a } {
	return [expr	  [mget $a 0 0] * [mget $a 1 1] * [mget $a 2 2] \
			+ [mget $a 0 1] * [mget $a 1 2] * [mget $a 2 0] \
			+ [mget $a 0 2] * [mget $a 1 0] * [mget $a 2 1] \
			- [mget $a 0 2] * [mget $a 1 1] * [mget $a 2 0] \
			- [mget $a 0 1] * [mget $a 1 0] * [mget $a 2 2] \
			- [mget $a 0 0] * [mget $a 1 2] * [mget $a 2 1] ]
	}

proc mat3subcyc { a i j } {
	return [list	[list	[mget $a [expr ($i + 1) % 3] [expr ($j + 1) % 3] ] \
				[mget $a [expr ($i + 1) % 3] [expr ($j + 2) % 3] ] ] \
			[list	[mget $a [expr ($i + 2) % 3] [expr ($j + 1) % 3] ] \
				[mget $a [expr ($i + 2) % 3] [expr ($j + 2) % 3] ] ] ]
	}

proc mat3invert { a } {
	set d [mat3det $a]
	if {$d == 0.0} {
		puts "mat3invert: cannot invert degenerate matrix $a"
		return $a
		}
	set adj	[list	[list	[expr ([mget $a 1 1] * [mget $a 2 2] - [mget $a 1 2] * [mget $a 2 1]) / double($d) ] \
				[expr ([mget $a 2 1] * [mget $a 0 2] - [mget $a 2 2] * [mget $a 0 1]) / double($d) ] \
				[expr ([mget $a 0 1] * [mget $a 1 2] - [mget $a 0 2] * [mget $a 1 1]) / double($d) ] ] \
			[list	[expr ([mget $a 1 2] * [mget $a 2 0] - [mget $a 1 0] * [mget $a 2 2]) / double($d) ] \
				[expr ([mget $a 2 2] * [mget $a 0 0] - [mget $a 2 0] * [mget $a 0 2]) / double($d) ] \
				[expr ([mget $a 0 2] * [mget $a 1 0] - [mget $a 0 0] * [mget $a 1 2]) / double($d) ] ] \
			[list	[expr ([mget $a 1 0] * [mget $a 2 1] - [mget $a 1 1] * [mget $a 2 0]) / double($d) ] \
				[expr ([mget $a 2 0] * [mget $a 0 1] - [mget $a 2 1] * [mget $a 0 0]) / double($d) ] \
				[expr ([mget $a 0 0] * [mget $a 1 1] - [mget $a 0 1] * [mget $a 1 0]) / double($d) ] ] ]
	return $adj
	}

proc mat3dual { a } {
	return	[list	[expr ([mget $a 1 2] - [mget $a 2 1]) / 2.0] \
			[expr ([mget $a 2 0] - [mget $a 0 2]) / 2.0] \
			[expr ([mget $a 0 1] - [mget $a 1 0]) / 2.0] ]
	}

proc mat3maptoquat { m } {
	set r [-vec3 [mat3dual $m]]
	set s [expr pow([mat3det $m],1.0/3.0)]
	set m [mat3/s $m $s]

	set a0q [expr ([mat3trace $m] + 1.0) / 4.0]
	set a1q [expr ([mget $m 0 0] - 2.0 * $a0q + 1.0) / 2.0]
	set a2q [expr ([mget $m 1 1] - 2.0 * $a0q + 1.0) / 2.0]
	set a3q [expr ([mget $m 2 2] - 2.0 * $a0q + 1.0) / 2.0]

	set a0 [expr sqrt(abs($a0q))]
	set a1 [expr [vget $r 0] < 0 ? [expr -sqrt(abs($a1q))] : [expr sqrt(abs($a1q))]]
	set a2 [expr [vget $r 1] < 0 ? [expr -sqrt(abs($a2q))] : [expr sqrt(abs($a2q))]]
	set a3 [expr [vget $r 2] < 0 ? [expr -sqrt(abs($a3q))] : [expr sqrt(abs($a3q))]]

	return [quat*s [list $a0 [list $a1 $a2 $a3]] [expr sqrt($s)]]
	}

proc mat3rotangles { r i j k } {
	append order $i $j $k
	
	if { $order == "012" } { set p 1 } \
	elseif { $order == "120" } { set p 1 } \
	elseif { $order == "201" } { set p 1 } \
	elseif { $order == "021" } { set p -1 } \
	elseif { $order == "210" } { set p -1 } \
	elseif { $order == "102" } { set p -1 } \
	else { puts {mat3rotangles: arguments i j k must be a permutation of 0 1 2} }

	if { [expr [mget $r $i $k] != 1] && [expr [mget $r $i $k] != -1] } {
		set wi [expr atan2(- [mget $r $j $k] * $p,[mget $r $k $k])]
		set wk [expr atan2(- [mget $r $i $j] * $p,[mget $r $i $i])]
		set wj [expr atan2(  [mget $r $i $k] * $p,sqrt([mget $r $j $k] * [mget $r $j $k] + [mget $r $k $k] * [mget $r $k $k]))]
		} \
	else {
		set wi 0.0
		set wj [expr atan2(  [mget $r $i $k] * $p,sqrt([mget $r $j $k] * [mget $r $j $k] + [mget $r $k $k] * [mget $r $k $k]))]
		set wk [expr atan2(  [mget $r $j $i] * $p,[mget $r $j $j])]
		}
	return [list $wi $wj $wk]
	}
	
proc mat3rotmatrix { i j k wi wj wk } {
	append order $i $j $k
	
	if { $order == "012" } { set p 1 } \
	elseif { $order == "120" } { set p 1 } \
	elseif { $order == "201" } { set p 1 } \
	elseif { $order == "021" } { set p -1 } \
	elseif { $order == "210" } { set p -1 } \
	elseif { $order == "102" } { set p -1 } \
	else { puts {mat3rotmatrix: arguments i j k must be a permutation of 0 1 2} }

	set ci [expr cos($wi)];set si [expr sin($wi) * $p];
	set cj [expr cos($wj)];set sj [expr sin($wj) * $p];
	set ck [expr cos($wk)];set sk [expr sin($wk) * $p];

	set r { { 0 0 0 } { 0 0 0 } { 0 0 0 } }

	mset r $i $i [expr $cj * $ck]
	mset r $i $j [expr - $cj * $sk]
	mset r $i $k [expr $sj]

	mset r $j $i [expr $si * $sj * $ck + $ci * $sk]
	mset r $j $j [expr - $si * $sj * $sk + $ci * $ck]
	mset r $j $k [expr - $si * $cj]

	mset r $k $i [expr - $ci * $sj * $ck + $si * $sk]
	mset r $k $j [expr $ci * $sj * $sk + $si * $ck]
	mset r $k $k [expr $ci * $cj]

	return $r
	}


proc ismat3 { a } {
	if { [llength $a] != 3 } {
		return 0
		}
	if { ! [isvec3 [mgetrow $a 0]] } {
		return 0
		}
	if { ! [isvec3 [mgetrow $a 1]] } {
		return 0
		}
	if { ! [isvec3 [mgetrow $a 2]] } {
		return 0
		}
	return 1
	}

# mat4.tcl

proc mat4*mat4 { a b } {
	return [list	[list	[expr	  [mget $a 0 0] * [mget $b 0 0] \
					+ [mget $a 0 1] * [mget $b 1 0] \
					+ [mget $a 0 2] * [mget $b 2 0] \
					+ [mget $a 0 3] * [mget $b 3 0] ] \
				[expr	  [mget $a 0 0] * [mget $b 0 1] \
					+ [mget $a 0 1] * [mget $b 1 1] \
					+ [mget $a 0 2] * [mget $b 2 1] \
					+ [mget $a 0 3] * [mget $b 3 1] ] \
				[expr	  [mget $a 0 0] * [mget $b 0 2] \
					+ [mget $a 0 1] * [mget $b 1 2] \
					+ [mget $a 0 2] * [mget $b 2 2] \
					+ [mget $a 0 3] * [mget $b 3 2] ] \
				[expr	  [mget $a 0 0] * [mget $b 0 3] \
					+ [mget $a 0 1] * [mget $b 1 3] \
					+ [mget $a 0 2] * [mget $b 2 3] \
					+ [mget $a 0 3] * [mget $b 3 3] ] ] \
			[list	[expr	  [mget $a 1 0] * [mget $b 0 0] \
					+ [mget $a 1 1] * [mget $b 1 0] \
					+ [mget $a 1 2] * [mget $b 2 0] \
					+ [mget $a 1 3] * [mget $b 3 0] ] \
				[expr	  [mget $a 1 0] * [mget $b 0 1] \
					+ [mget $a 1 1] * [mget $b 1 1] \
					+ [mget $a 1 2] * [mget $b 2 1] ] \
				[expr	  [mget $a 1 0] * [mget $b 0 2] \
					+ [mget $a 1 1] * [mget $b 1 2] \
					+ [mget $a 1 2] * [mget $b 2 2] \
					+ [mget $a 1 3] * [mget $b 3 2] ] \
				[expr	  [mget $a 1 0] * [mget $b 0 3] \
					+ [mget $a 1 1] * [mget $b 1 3] \
					+ [mget $a 1 2] * [mget $b 2 3] \
					+ [mget $a 1 3] * [mget $b 3 3] ] ] \
			[list	[expr	  [mget $a 2 0] * [mget $b 0 0] \
					+ [mget $a 2 1] * [mget $b 1 0] \
					+ [mget $a 2 2] * [mget $b 2 0] \
					+ [mget $a 2 3] * [mget $b 3 0] ] \
				[expr	  [mget $a 2 0] * [mget $b 0 1] \
					+ [mget $a 2 1] * [mget $b 1 1] \
					+ [mget $a 2 2] * [mget $b 2 1] \
					+ [mget $a 2 3] * [mget $b 3 1] ] \
				[expr	  [mget $a 2 0] * [mget $b 0 2] \
					+ [mget $a 2 1] * [mget $b 1 2] \
					+ [mget $a 2 2] * [mget $b 2 2] \
					+ [mget $a 2 3] * [mget $b 3 2] ] \
				[expr	  [mget $a 2 0] * [mget $b 0 3] \
					+ [mget $a 2 1] * [mget $b 1 3] \
					+ [mget $a 2 2] * [mget $b 2 3] \
					+ [mget $a 2 3] * [mget $b 3 3] ] ] \
			[list	[expr	  [mget $a 3 0] * [mget $b 0 0] \
					+ [mget $a 3 1] * [mget $b 1 0] \
					+ [mget $a 3 2] * [mget $b 2 0] \
					+ [mget $a 3 3] * [mget $b 3 0] ] \
				[expr	  [mget $a 3 0] * [mget $b 0 1] \
					+ [mget $a 3 1] * [mget $b 1 1] \
					+ [mget $a 3 2] * [mget $b 2 1] \
					+ [mget $a 3 3] * [mget $b 3 1] ] \
				[expr	  [mget $a 3 0] * [mget $b 0 2] \
					+ [mget $a 3 1] * [mget $b 1 2] \
					+ [mget $a 3 2] * [mget $b 2 2] \
					+ [mget $a 3 3] * [mget $b 3 2] ] \
				[expr	  [mget $a 3 0] * [mget $b 0 3] \
					+ [mget $a 3 1] * [mget $b 1 3] \
					+ [mget $a 3 2] * [mget $b 2 3] \
					+ [mget $a 3 3] * [mget $b 3 3] ] ] ] \
	}

#neu:
proc srt=vecmat { v m } {
	return [list $v $m]
	}

# srt2.tcl

proc srt2=vec2mat2 { v m } {
	return [list $v $m]
	}

proc srt2*vec2 { a v } {
	return [vec2+vec2 [mat2*vec2 [srtgetmat $a] $v] [srtgetvec $a]]
	}

proc srt2*mat2 { a m } {
	return [mat2*mat2 [srtgetmat $a] $m]
	}

proc srt2*srt2 { a b } {
	return [list [vec2+vec2 [srtgetvec $a] [mat2*vec2 [srtgetmat $a] [srtgetvec $b]]] [mat2*mat2 [srtgetmat $a] [srtgetmat $b]]]
	}

proc srt2invert { a } {
	set i [mat2invert [srtgetmat $a]]
	return [list [-vec2 [mat2*vec2 $i [srtgetvec $a]]] $i]
	}

proc mat3=srt2 { a } {
	set m [srtgetmat $a]

	set c0 [mgetrow $m 0]
	set c1 [mgetrow $m 1]
	set c2 { 0 0 1 }

	set v [srtgetvec $a]

	lappend c0 [vget $v 0]
	lappend c1 [vget $v 1]

	return [list $c0 $c1 $c2]
	}

# srt3.tcl

#neu
proc srt3=vec3mat3 { v m } {
	return [list $v $m]
	}

proc srt3*vec3 { a v } {
	return [vec3+vec3 [mat3*vec3 [srtgetmat $a] $v] [srtgetvec $a]]
	}

proc srt3*mat3 { a m } {
	return [mat3*mat3 [srtgetmat $a] $m]
	}

proc srt3*srt3 { a b } {
	return [list [vec3+vec3 [srtgetvec $a] [mat3*vec3 [srtgetmat $a] [srtgetvec $b]]] [mat3*mat3 [srtgetmat $a] [srtgetmat $b]]]
	}

proc srt3invert { a } {
	set i [mat3invert [srtgetmat $a]]
	return [list [-vec3 [mat3*vec3 $i [srtgetvec $a]]] $i]
	}

proc mat4=srt3 { a } {
	set m [srtgetmat $a]

	set c0 [mgetrow $m 0]
	set c1 [mgetrow $m 1]
	set c2 [mgetrow $m 2]
	set c3 { 0 0 0 1 }

	set v [srtgetvec $a]

	lappend c0 [vget $v 0]
	lappend c1 [vget $v 1]
	lappend c2 [vget $v 2]

	return [list $c0 $c1 $c2 $c3]
	}









